//----------------------------------------------------------------------------------
// File: Geometry.h
// Author: Mathias Schott
// Email:  sdkfeedback@nvidia.com
// 
// Copyright (c) 2007 NVIDIA Corporation. All rights reserved.
//
// TO  THE MAXIMUM  EXTENT PERMITTED  BY APPLICABLE  LAW, THIS SOFTWARE  IS PROVIDED
// *AS IS*  AND NVIDIA AND  ITS SUPPLIERS DISCLAIM  ALL WARRANTIES,  EITHER  EXPRESS
// OR IMPLIED, INCLUDING, BUT NOT LIMITED  TO, IMPLIED WARRANTIES OF MERCHANTABILITY
// AND FITNESS FOR A PARTICULAR PURPOSE.  IN NO EVENT SHALL  NVIDIA OR ITS SUPPLIERS
// BE  LIABLE  FOR  ANY  SPECIAL,  INCIDENTAL,  INDIRECT,  OR  CONSEQUENTIAL DAMAGES
// WHATSOEVER (INCLUDING, WITHOUT LIMITATION,  DAMAGES FOR LOSS OF BUSINESS PROFITS,
// BUSINESS INTERRUPTION, LOSS OF BUSINESS INFORMATION, OR ANY OTHER PECUNIARY LOSS)
// ARISING OUT OF THE  USE OF OR INABILITY  TO USE THIS SOFTWARE, EVEN IF NVIDIA HAS
// BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
//
//
//----------------------------------------------------------------------------------


#pragma once
#include "Utility.h"
#include <vector>
namespace Utility
{

namespace Geometry
{

template<typename V>
class SimpleVertexBuffer
{
public:

	SimpleVertexBuffer(ID3D10Device* device, const std::vector<V>& vertices, D3D10_USAGE usage =  D3D10_USAGE_DEFAULT, UINT bindflags = D3D10_BIND_VERTEX_BUFFER, UINT cpu_flags = 0):
		m_device(device),
		m_buffer(0),
		m_num_vertices(UINT(vertices.size()))
	{
		HRESULT hr;
		m_device->AddRef();
		
		D3D10_BUFFER_DESC bd;
		bd.Usage = usage ;
		bd.ByteWidth = sizeof( V ) * UINT(vertices.size());
		bd.BindFlags = bindflags;
		bd.CPUAccessFlags = cpu_flags;
		bd.MiscFlags = 0;

		D3D10_SUBRESOURCE_DATA InitData;
		InitData.pSysMem = &vertices[0];

		V(device->CreateBuffer( &bd, &InitData, &m_buffer ));

		m_stride = sizeof( V );
		m_offset = 0;
	}

	~SimpleVertexBuffer()
	{
		m_buffer->Release();
		m_device->Release();
		
	}

	UINT NumVertices() const
	{
		return m_num_vertices;
	}
	
	ID3D10Buffer*	Buffer()
	{
		return m_buffer;

	}

	void BindToInputAssembler()
	{
		m_device->IASetVertexBuffers( 0, 1, &m_buffer, &m_stride, &m_offset );
	}

	void BindToStreamOut()
	{
		UINT offset[1] = { 0 };
		m_device->SOSetTargets( 1,  &m_buffer, offset);
	}

protected:
private:
	UINT		m_num_vertices;
	

	ID3D10Device*	m_device;

	ID3D10Buffer*	m_buffer;

	UINT			m_stride;
	UINT			m_offset;


};

ID3D10InputLayout* CreateInputLayout(ID3D10Device* device, const std::vector<D3D10_INPUT_ELEMENT_DESC>& layout, ID3D10EffectTechnique* technique);
	

} // Geometry

}// Utility

