//----------------------------------------------------------------------------------
// File:   Scene.fx
// Author: Mathias Schott
// Email:  sdkfeedback@nvidia.com
// 
// Copyright (c) 2007 NVIDIA Corporation. All rights reserved.
//
// TO  THE MAXIMUM  EXTENT PERMITTED  BY APPLICABLE  LAW, THIS SOFTWARE  IS PROVIDED
// *AS IS*  AND NVIDIA AND  ITS SUPPLIERS DISCLAIM  ALL WARRANTIES,  EITHER  EXPRESS
// OR IMPLIED, INCLUDING, BUT NOT LIMITED  TO, IMPLIED WARRANTIES OF MERCHANTABILITY
// AND FITNESS FOR A PARTICULAR PURPOSE.  IN NO EVENT SHALL  NVIDIA OR ITS SUPPLIERS
// BE  LIABLE  FOR  ANY  SPECIAL,  INCIDENTAL,  INDIRECT,  OR  CONSEQUENTIAL DAMAGES
// WHATSOEVER (INCLUDING, WITHOUT LIMITATION,  DAMAGES FOR LOSS OF BUSINESS PROFITS,
// BUSINESS INTERRUPTION, LOSS OF BUSINESS INFORMATION, OR ANY OTHER PECUNIARY LOSS)
// ARISING OUT OF THE  USE OF OR INABILITY  TO USE THIS SOFTWARE, EVEN IF NVIDIA HAS
// BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
//
//
//----------------------------------------------------------------------------------

// uniforms
cbuffer PerFrame
{
	matrix world;
	matrix view;
	matrix projection;
	
	matrix world_view;
	matrix world_view_projection;
	
	float time;
}

struct SceneApp2VS
{
    float3 Position : Position;
    float3 Normal   : Normal;
    float2 TexCoord : TexCoord0;
};

struct SceneVS2PS
{
    float4 Position : SV_Position;
    float2 TexCoord : TexCoord0;
    float4 Color : Color;
};

Texture2D	tex_diffuse;

SamplerState LinearSample 
{
	Filter = Min_Mag_Mip_Linear;
    AddressU = Wrap;
    AddressV = Wrap;
};

RasterizerState EnableCulling
{
    CullMode = Back;
    MultiSampleEnable = true;
};

BlendState NoBlending
{
    BlendEnable[0] = false;
};

DepthStencilState EnableDepthTestWrite
{
    DepthEnable = true;
    DepthWriteMask = All;
};

void SceneVS( SceneApp2VS input, out SceneVS2PS output )
{
    float3 world_position = mul( float4(input.Position, 1.0f), world );
	float3 world_normal = normalize(mul(world, float4(input.Normal,0)).xyz);
	
	float3 light_position = float3(0, 20,0);
	float3 light_dir = normalize(light_position - world_position);
	
	output.Position = mul( float4(input.Position, 1.0f), world_view_projection );
    output.Color = 0.3f + saturate(dot(world_normal, light_dir));
    output.TexCoord = input.TexCoord;
    
}

void ScenePS(SceneVS2PS input, out float4 output : SV_Target) 
{   
	output = tex_diffuse.Sample(LinearSample,input.TexCoord) * input.Color;
}

technique10 RenderScene
{
    pass p0
    {
        SetVertexShader( CompileShader( vs_4_0, SceneVS() ) );
        SetGeometryShader( NULL );
        SetPixelShader( CompileShader( ps_4_0, ScenePS() ) );
        
        SetRasterizerState( EnableCulling );
        SetDepthStencilState( EnableDepthTestWrite, 0 );
        SetBlendState(NoBlending, float4(1.0f, 1.0f, 1.0f, 1.0f), ~0);
    }  
}
